@@ -189,6 +189,7 @@ class U2idx(object):
         is_key = True
         is_size = False
         is_date = False
+        is_wark = False
         field_end = ""  # closing parenthesis or whatever
         kw_key = ["(", ")", "and ", "or ", "not "]
         kw_val = ["==", "=", "!=", ">", ">=", "<", "<=", "like "]
@@ -207,6 +208,8 @@ class U2idx(object):
                     is_key = kw in kw_key
                     uq = uq[len(kw) :]
                     ok = True
+                    if is_wark:
+                        kw = "= "
                     q += kw
                     break
 
@@ -249,6 +252,10 @@ class U2idx(object):
                     if icase:
                         v = "casefold(%s)" % (v,)
 
+                elif v == "w":
+                    v = "substr(up.w,1,16)"
+                    is_wark = True
+
                 elif v == "tags" or ptn_mt.match(v):
                     have_mt = True
                     field_end = ") "
@@ -260,7 +267,7 @@ class U2idx(object):
                     v = "exists(select 1 from mt where mt.w = mtw and " + vq
 
                 else:
-                    raise Pebkac(400, "invalid key [{}]".format(v))
+                    raise Pebkac(400, "invalid key %r" % (v,))
 
                 q += v + " "
                 continue
@@ -289,6 +296,14 @@ class U2idx(object):
                 is_size = False
                 v = int(float(v) * 1024 * 1024)
 
+            elif is_wark:
+                is_wark = False
+                v = v.strip("*")
+                if len(v) > 16:
+                    v = v[:16]
+                if len(v) < 16:
+                    raise Pebkac(400, "w/filehash must be 16+ chars")
+
             else:
                 if v.startswith("*"):
                     head = "'%'||"
